xquery version "3.1";

(:~
 : This module provides the API endpoint for kmg-print.
 :
 : @version 1.5 (2024-11-05)
 : @status stable
 : @author Max-Ferdinand Zeterberg
 :)
module namespace rest-print="https://mollenhauer-edition.de/ns/rest-print";

import module namespace config="https://sade.textgrid.de/ns/config" at "../../config.xqm";
import module namespace selection = "https://mollenhauer-edition.de/ns/conversions/selection" at "selection.xqm";
import module namespace simplify="https://mollenhauer-edition.de/ns/conversions/simplify" at "simplify.xqm";
import module namespace rest="http://exquery.org/ns/restxq";
import module namespace removal="https://mollenhauer-edition.de/ns/conversions/removal" at "removal.xqm";
import module namespace studienausgabe="https://mollenhauer-edition.de/ns/conversions/studienausgabe" at "studienausgabe.xqm";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace output ="http://www.w3.org/2010/xslt-xquery-serialization";

(: ~
 : This functions allows to request data for print publication.
 :
 : @param $kmg KMG-Number
 : @return A XML object (simplified TEI-XML)
 :)
declare
    %rest:path("api/kmg4sta/{$TGURI}/{$fassung}")
    %rest:GET
    %rest:form-param("withoutcommentary", "{$withoutcommentary}")
    %output:method("xml")
function rest-print:kmg4sta($TGURI as xs:string*, $fassung as xs:string*, $withoutcommentary as xs:string*) as item()*
{
    try {
        let $doc := doc($config:tei || $TGURI || '.xml')
        let $versionID := (
            if ( empty($fassung) ) then
                data($doc//tei:sourceDesc/tei:listWit/tei:witness[last()]/@xml:id)
            else
                if ( $doc//tei:sourceDesc/tei:listWit/tei:witness[@xml:id = $fassung] ) then
                    $fassung
                else
                    'error'
        )
        return
            if ( $versionID = 'error' ) then
                    element error {
                        attribute n {10},
                        'Document with TGURI ' || $TGURI || ' has no Fassung' || $fassung || '.'
                    }
            else
                let $afterCommentary := (
                    if ( ($withoutcommentary) ) then
                        removal:remove-commentary($doc)
                    else
                        $doc
                    )
                let $removedWitnesses := removal:remove-witnesses($afterCommentary, $versionID)
                let $single := simplify:all($removedWitnesses, $versionID)
                let $removedsic := removal:delete-sic-studienausgabe($single)
                let $bibls := studienausgabe:all-citations($removedsic)
                let $newDoc := studienausgabe:new-doc($removedsic, $bibls)
                return
                    $newDoc
    } catch * {
        element error {
            attribute n {9},
            'Document with TGURI ' || $TGURI || ' could not retrieved for unknown reason.'
        }
    }
};