xquery version "3.1";

module namespace editorial="https://mollenhauer-edition.de/ns/editorial";

import module namespace config="https://sade.textgrid.de/ns/config" at "../../config.xqm";
import module namespace functx="http://www.functx.com";
import module namespace lang="https://sade.textgrid.de/ns/lang" at "../../lang.xqm";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace html="http://www.w3.org/1999/xhtml";
declare namespace tg="http://textgrid.info/namespaces/metadata/core/2010";

declare function editorial:note($note as node()*)
as element(div)?
{
    element div {
        $note/@*,
        element h4 {
            attribute class { "editorial-content-title" },
            lang:translate('editorialNote'),
            element button {
                attribute class {"button-close index-entry-dismiss"},
                attribute aria-label {"Close"},
                element span {
                    attribute class {"icon-inverse icon-xmark"},
                    attribute aria-hidden {"true"}
                }
            }
        },
        element div {
            attribute class { "editorial-content-body" },
            $note/node()
        }
    }
};

declare function editorial:correction($correction as node()*)
as element(div)?
{
    element div {
        $correction/@*,
        element h4 {
            attribute class { "editorial-content-title" },
            lang:translate('editorial_correction'),
            element button {
                attribute class {"button-close index-entry-dismiss"},
                attribute aria-label {"Close"},
                element span {
                    attribute class {"icon-inverse icon-xmark"},
                    attribute aria-hidden {"true"}
                }
            }
        },
        element div {
            attribute class { "editorial-content-body" },
            $correction/node()
        }
    }
};

declare function editorial:addition($addition as node()*, $type as xs:string)
as element(div)?
{
    element div {
        $addition/@*,
        element h4 {
            attribute class { "editorial-content-title" },
            lang:translate(
                switch ($type)
                    case "highlight" return 'highlight'
                    default return 'addition'
            ),
            element button {
                attribute class {"button-close index-entry-dismiss"},
                attribute aria-label {"Close"},
                element span {
                    attribute class {"icon-inverse icon-xmark"},
                    attribute aria-hidden {"true"}
                }
            }
        },
        element div {
            attribute class { "editorial-content-body" },
            lang:translate("by"),
            " ",
            $addition/node()
        }
    }
};

declare function editorial:bibliographic-reference($reference as node()*)
as element(div)?
{
    element div {
        $reference/@*,
        attribute data-resolved { "false" },
        element h4 {
            attribute class { "editorial-content-title" },
            if ($reference[matches(@class, "bibliographic-reference-added")]) then
                lang:translate('biblReferenceAdd')
            else
                lang:translate('biblReference'),
            element button {
                attribute class {"button-close index-entry-dismiss"},
                attribute aria-label {"Close"},
                element span {
                    attribute class {"icon-inverse icon-xmark"},
                    attribute aria-hidden {"true"}
                }
            }
        },
        element div {
            attribute class { "editorial-content-body" },
            element div {
                attribute class { "load-error hidden" },
                attribute aria-hidden { "true" },
                lang:translate('loadError')
            },
            if ($reference/@data-checked = "true") then
                element div {
                    lang:translate('citControl'),
                    element a {
                        attribute href { "https://www.zotero.org/groups/2206325/kmg-pool/items/" || $reference/@data-ref => replace("zotero:", "") },
                        attribute target { "_blank" },
                        " ",
                        $reference/@data-ref => replace("zotero:", "")
                    }
                }
            else
                ()
        }
    }
};

declare function editorial:internal-bibliographic-reference($reference as node()*)
as element(div)?
{
    element div {
        $reference/@*,
        element h4 {
            attribute class { "editorial-content-title" },
                lang:translate('biblReference'),
            element button {
                attribute class {"button-close index-entry-dismiss"},
                attribute aria-label {"Close"},
                element span {
                    attribute class {"icon-inverse icon-xmark"},
                    attribute aria-hidden {"true"}
                }
            }
        },
        element div {
            attribute class { "editorial-content-body" },
            $reference/node()/node()/node()
        }
    }
};

declare function editorial:abbreviation($abbreviation as node()*)
as element(div)?
{
    element div {
        $abbreviation/@*,
        element h4 {
            attribute class { "editorial-content-title" },
            lang:translate('abbreviation'),
            element button {
                attribute class {"button-close index-entry-dismiss"},
                attribute aria-label {"Close"},
                element span {
                    attribute class {"icon-inverse icon-xmark"},
                    attribute aria-hidden {"true"}
                }
            }
        },
        element div {
            attribute class { "editorial-content-body" },
            $abbreviation/node()
        }
    }
};

declare function editorial:musical-work($work as node()*)
as element(div)?
{
    let $work-ref := $work/@data-target
    let $id := substring-after($work-ref, ":")
    return
        element div {
            $work/@*,
            element h4 {
                attribute class { "editorial-content-title" },
                lang:translate('musicalWork'),
                element button {
                    attribute class {"button-close index-entry-dismiss"},
                    attribute aria-label {"Close"},
                    element span {
                        attribute class {"icon-inverse icon-xmark"},
                        attribute aria-hidden {"true"}
                    }
                }
            },
            element dl {
                attribute class { "index-entry-body" },
                element dt {
                    attribute class { "index-entry-label" },
                    lang:translate('title')
                },
                element dd {
                    attribute class { "index-entry-value" },
                    $work/@data-title/string() => normalize-space()
                },
                if ($work-ref = "") then
                    ()
                else
                    (element dt {
                        attribute class { "index-entry-label" },
                        lang:translate('gndEntry')
                    },
                    element dd {
                        attribute class { "index-entry-value" },
                        element a {
                            attribute href { "http://d-nb.info/gnd/" || $id },
                            attribute target { "_blank" },
                            "http://d-nb.info/gnd/" || $id,
                            " ",
                            element span {
                                attribute class { "fa fa-external-link-alt" }
                            }
                        }
                    }),
                element div {
                    attribute class { "editorial-external-media" },
                    $work/node()
                },
                element dt {
                    attribute class { "index-entry-label" },
                    lang:translate('recordingSource')
                },
                element dd {
                    attribute class { "index-entry-value" },
                    element a {
                        attribute href { $work/@data-source },
                        attribute target { "_blank" },
                        replace($work/@data-source, "http[s]://", "") => substring-before("/"),
                        " ",
                        element span {
                            attribute class { "fa fa-external-link-alt" }
                        }
                    }
                },
                element span {
                    attribute class { "index-entry-index-link" },
                    element a {
                        attribute href {
                            "register.html?entity="
                            || $id
                            || "&amp;register=music"
                        },
                        attribute target { "_blank" },
                        (lang:translate("indexlink")),
                        " ",
                        element span {
                            attribute class { "fa fa-external-link-alt" }
                        }
                    }
                }
            }
        }
};