xquery version "3.1";

module namespace kmg-register="https://mollenhauer-edition.de/ns/register";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace xhtml="http://www.w3.org/1999/xhtml";
declare namespace index="https://mollenhauer-edition.de/index";

import module namespace config="https://sade.textgrid.de/ns/config" at "../config.xqm";
import module namespace transformKMG="http://mollenhauer-edition.de/ns/transform" at "transform.xqm";
import module namespace kmg-util="http://mollenhauer-edition.de/ns/util" at "kmg-util.xqm";
import module namespace lang="https://sade.textgrid.de/ns/lang" at "../lang.xqm";

declare function kmg-register:get-selector($node as node(), $model as map(*))
{
    let $selection := request:get-parameter("register", "person")
    let $registers := (
        "person",
        "art",
        "music",
        "brand",
        "org",
        "periodica")
    let $lang := request:get-parameter("lang", "de")
    return
        for $register in $registers return
            element a {
                attribute href { "register.html?register=" || $register },
                if ($register eq $selection) then
                    attribute class { "selected" }
                else
                    (),
                lang:translate($register || "Register")
            }
};

declare function kmg-register:get-registers($node as node(), $model as map(*))
{
    let $selection := request:get-parameter("register", "person")
    let $lang := request:get-parameter("lang", "de")
    let $doc := doc($config:register || $selection || ".xml")
    return
        kmg-register:get-register($doc, $lang, $selection)
};

declare function kmg-register:get-register($doc as node()*, $lang as xs:string, $selection as xs:string)
{
    let $register :=
        if ($doc/index:index/@type) then
            $doc/index:index/@type
        else
            "person"
    return
        (element noscript
        {
            element h2
            {
                switch ($doc/index:index/@type)
                case "person"
                return
                    "Personenregister"
                case "art"
                return
                    "Register der Kunstwerke"
                case "music"
                return
                    "Register der Musikwerke"
                default
                return
                    "Register"
            }
        },
        element ol
        {
            attribute class { "index-entries" },
            attribute data-indextype { $register },
            attribute role { "list" },
            kmg-register:get-entries($doc, $lang, $register)
        })
};

declare function kmg-register:get-entries($doc as node()*, $lang as xs:string, $index as xs:string)
{
    let $requested := request:get-parameter("entity", "")
    return
    for $entry in kmg-register:sort-entries($doc, $lang, $index)
    let $ref := $entry/@index:ref => substring-after(":")
    return
        element li
        {
            attribute class { "index-entry" },
            if ($index = "art") then
                attribute data-letter { substring($entry/index:name[@xml:lang = $lang][1]/text(), 1, 1) => lower-case()}
            else
                attribute data-letter { substring($entry/index:name/text(), 1, 1) => lower-case()},
            attribute data-target {
                if ($entry/@index:ref => starts-with("textgrid:")) then
                    $entry/@index:ref => substring-after("#")
                else
                    $entry/@index:ref => substring-after(":")
            },
            element details
            {
                if ($requested = $entry/@index:ref => substring-after(":")) then
                    attribute open {}
                else
                    (),
                attribute id {
                    if ($entry/@index:ref => starts-with("textgrid:")) then
                        if ($index = "art") then
                            $entry/@index:ref => substring-after(":")
                        else
                            $entry/@index:ref => substring-after("#")
                    else
                        $entry/@index:ref => substring-after(":")
                 },
                attribute name { "register" },
                element summary {
                    attribute class { "index-entry-head" },
                    if ($index = "art") then
                        $entry/index:name[@xml:lang = $lang]/text()
                    else
                        $entry/index:name/text()
                },
                element dl {
                    attribute class { "index-entry-body" },
                    if ($entry/index:artist) then
                        kmg-register:multi-entry(lang:translate("indexartist"), $entry/index:artist)
                    else
                        (),
                    if ($entry/index:reproduction) then
                        for $reproduction in $entry/index:reproduction return
                        kmg-register:make-link(lang:translate("indexreproduction"), $reproduction)
                    else
                        (),
                    if ($entry/index:birth) then
                        kmg-register:make-entry(
                            lang:translate("indexbirth"),
                            try {
                                xs:date($entry/index:birth/text()) => format-date("[D]. [MNn] [Y]", "de", (), ())
                            }
                            catch * {
                                $entry/index:birth/text()
                            })
                    else
                        (),
                    if ($entry/index:death) then
                        kmg-register:make-entry(
                            lang:translate("indexdeath"),
                            try {
                                xs:date($entry/index:death/text()) => format-date("[D]. [MNn] [Y]", "de", (), ())
                            }
                            catch * {
                                $entry/index:death/text()
                            })
                    else
                        (),
                    if ($entry/index:udk) then
                        kmg-register:make-entry(lang:translate("indexudk"), $entry/index:udk/text())
                    else
                        (),
                    if ($entry/index:category) then
                        kmg-register:multi-entry(lang:translate("indexcategory"), $entry/index:category)
                    else
                        (),
                    if ($entry/index:date) then
                        kmg-register:make-entry(lang:translate("indexdate"), $entry/index:date/text())
                    else
                        (),
                    if ($entry/index:place) then
                        kmg-register:make-entry(lang:translate("indexplace"), $entry/index:place/text())
                    else
                        (),
                    if ($entry/index:measurements) then
                        kmg-register:make-entry(lang:translate("indexmeasurements"), $entry/index:measurements/text())
                    else
                        (),
                    if ($entry/index:material) then
                        kmg-register:make-entry(lang:translate("indexmaterial"), $entry/index:material/text())
                    else
                        (),
                    if ($entry/index:repository) then
                        if ($entry/index:repository/@index:ref) then
                            kmg-register:make-link(lang:translate("indexrepository"), $entry/index:repository)
                        else
                            kmg-register:make-entry(lang:translate("indexrepository"), $entry/index:repository/text())
                    else
                        (),
                    if ($entry/index:workId) then
                        kmg-register:make-entry(lang:translate("indexworkId"), $entry/index:workId/text())
                    else
                        (),
                    if ($entry/index:keyword) then
                        kmg-register:list-entry(lang:translate("indexkeyword"), $entry/index:keyword)
                    else
                        (),
                    if ($entry/index:external) then
                        kmg-register:list-entry(lang:translate("indexexternal"), $entry/index:external)
                    else
                        (),
                    if ($index = "art") then
                        ()
                    else
                        if ($ref) then
                            if ($entry/@index:source = "kmg") then
                                element dt {
                                lang:translate("indexNoSource")}
                            else
                                (element dt {
                                    attribute class { "index-entry-label index-entry-source" },
                                    lang:translate("indexsource")
                                },
                                    element dd {
                                        attribute class { "index-entry-value" },
                                        element a {
                                            attribute href { "http://d-nb.info/gnd/" || $ref },
                                            attribute target { "_blank" },
                                            "http://d-nb.info/gnd/" || $ref,
                                            " ",
                                            element span {
                                                attribute class { "fa fa-external-link-alt" }
                                            }
                                        }
                                    })
                        else
                            (),
                    if ($entry/index:references/*) then
                        (element dt {
                            attribute class { "index-entry-label" },
                            lang:translate("indexreferences")
                        },
                        element dd {
                            attribute class { "index-entry-value" },
                            element ul {
                                attribute role { "list" },
                                for $reference in $entry//index:reference return
                                    let $doc := doc($config:tei || $reference/@index:ref)
                                    let $title := string-join(transformKMG:transform($doc//tei:title[@type="main"], $reference/@index:ref, "index", "", ""), " ")
                                    let $mainWit := kmg-util:getLeittextID($reference/@index:ref)
                                    let $view :=
                                        if ($reference/@index:view = "edition") then
                                            $mainWit
                                        else if ($reference/@index:view = "commentary") then
                                            "commentary"
                                        else
                                            "both"
                                    let $id :=
                                        if ($entry/@index:source = "kmg") then
                                            $entry/@index:ref => substring-after("#")
                                        else
                                            $entry/@index:ref
                                    let $text := $reference/@index:ref => replace(".xml", "")
                                    return
                                        if ($view = "both") then
                                            (kmg-register:link-to-edition($text, $mainWit, $id, $title),
                                            kmg-register:link-to-edition($text, "commentary", $id, $title))
                                        else
                                            kmg-register:link-to-edition($text, $view, $id, $title)
                            }
                        })
                    else
                        ()
                }
            }
        }
};

declare function kmg-register:link-to-edition($uri as xs:string, $view as xs:string, $id as xs:string, $title as xs:string)
as element(li)
{
    element li {
        element a {
            attribute href {
                "kmg.html?file="
                || $uri
                || "&amp;edition=" || $view
                || "&amp;entity=" || $id
                },
            attribute target { "_blank" },
            $title,
            " ",
            if($view = "commentary") then
                ("(" || lang:translate("commentary") || ") ")
            else (),
            element span {
                attribute class { "fa fa-external-link-alt" }
            }
        }
    }
};

declare function kmg-register:multi-entry($label as xs:string, $values as node()*)
{
    element dt {
        attribute class { "index-entry-label" },
        $label
    },
    element dd {
        attribute class { "index-entry-value" },
        element ul {
            attribute role { "list" },
            for $value in $values return
                element li {
                    if ($value/@index:ref) then
                        element a {
                        attribute href { $value/@index:ref },
                        attribute target { "_blank" },
                        $value/text(),
                        " ",
                        element span {
                            attribute class { "fa fa-external-link-alt" }
                        }
                    }
                    else
                        $value/text()
                }
        }
    }
};

declare function kmg-register:list-entry($label as xs:string, $values as node()*)
{
    element dt {
        attribute class { "index-entry-label" },
        $label
    },
    element dd {
        attribute class { "index-entry-value" },
        element ul {
            attribute role { "list" },
            for $value in $values return
                element li {
                    if ($value/@index:ref) then
                        element a {
                            attribute href { $value/@index:ref },
                            attribute target { "_blank" },
                            $value/text(),
                            " ",
                            element span {
                                attribute class { "fa fa-external-link-alt" }
                            }
                        }
                    else
                        $value/text()
                }
        }
    }
};

declare function kmg-register:make-entry($label as xs:string?, $value as xs:string?)
{
    if ($label and $value) then
        (element dt {
            attribute class { "index-entry-label" },
            $label
        },
        element dd {
            attribute class { "index-entry-value" },
            $value
        })
    else ()
};

declare function kmg-register:make-link($label as xs:string, $value as node())
{
    if ($label and $value) then
        (element dt {
            attribute class { "index-entry-label" },
            $label
        },
        element dd {
            attribute class { "index-entry-value" },
            element a {
                attribute href { $value/@index:ref },
                attribute target { "_blank" },
                $value/text(),
                " ",
                element span {
                    attribute class { "fa fa-external-link-alt" }
                }
            }
        })
    else ()
};

declare function kmg-register:sort-entries($doc as node()*, $lang as xs:string, $index as xs:string)
{
    if ($index = "art") then
        for $entry in $doc/index:index/index:body/index:entry
        order by $entry/index:name[@xml:lang = $lang][1]/text()
        return
            $entry
    else
        for $entry in $doc/index:index/index:body/index:entry
        where substring($entry/index:name/text(), 1, 1) ne "_"
        where $entry/index:name != ""
        order by $entry/@index:sortName
        return
            $entry
};