xquery version "3.1";

module namespace search="http://mollenhauer-edition.de/ns/search";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace html="http://www.w3.org/1999/xhtml";
declare namespace index="https://mollenhauer-edition.de/index";

import module namespace config="https://sade.textgrid.de/ns/config" at "../../config.xqm";
import module namespace kwic="http://exist-db.org/xquery/kwic";

import module namespace facets="http://mollenhauer-edition.de/ns/facets" at "facets.xqm";
import module namespace options="http://mollenhauer-edition.de/ns/options" at "options.xqm";
import module namespace lang="https://sade.textgrid.de/ns/lang" at "../../lang.xqm";
import module namespace kmg-util="http://mollenhauer-edition.de/ns/util" at "../kmg-util.xqm";
import module namespace contentapi="http://sepia.io/content-parser/content-api" at "/db/system/repo/content-parser-1.5.2/content/content-api.xqm";

declare function search:search-area($term as xs:string*) {
    let $search := search:search($term)
    return
        element div {
            attribute class {"search-area"},
            element div {
                attribute class {"search-params"},
                options:get-options($search),
                facets:get-facets($search)
            },
            element div {
                attribute class {"search-results"},
                search:pagination($search),
                search:results($search, $term)
            }
        }

};

declare function search:pagination($search)
{
    let $count := count(
        for $hits in $search return
        for $hit in (kwic:summarize($hits, <config width="1"/>)) return $hit
    )
    let $text-count := count($search)
    let $text-hits := lang:translate("hits") || " in"
    let $text-texts := if ($text-count eq 1) then lang:translate("textSg") else lang:translate("textPlDat")
    let $sort := search:get-sorting()
    return
    element div {
        attribute class { "search-pagination" },
        element span {
            attribute class { "search-pagination-count" },
            $count, $text-hits, $text-count, $text-texts
        },
        element label {
            attribute class { "search-sort" },
            lang:translate("sortBy") || ": ",
            element select {
                attribute class {"search-sort-input"},
                attribute name {"sort"},
                attribute onchange {"this.form.submit()"},
                element option {
                    attribute value {"relevance"},
                    if ($sort eq "relevance") then
                        attribute selected {"selected"}
                    else (),
                    lang:translate("relevance")
                },
                element option {
                    attribute value {"dateAsc"},
                    if ($sort eq "dateAsc") then
                        attribute selected {"selected"}
                    else (),
                    lang:translate("sortChronAsc")
                },
                element option {
                    attribute value {"dateDesc"},
                    if ($sort eq "dateDesc") then
                        attribute selected {"selected"}
                    else (),
                    lang:translate("sortChronDesc")
                }
            }
        }
    }
};

declare function search:results($search, $term as xs:string)
{
    element ol {
        attribute class {"search-list search-results"},
        attribute role {"list"},
        let $sortParameter := search:get-sorting()
        let $direction := if ($sortParameter = "dateAsc") then "ascending" else "descending"
        return
            if ($direction eq "ascending") then
                search:search-ascending($search, $term, $direction)
            else
                search:search-descending($search, $term, $direction)
    }
};

declare function search:search-ascending($search, $term, $sort-parameter) {
    for $hits in $search
        let $doc-name := util:document-name($hits)
        let $uri := ft:field($hits, 'uri')
        let $idno := ft:field($hits, 'idno')
        let $doc := doc($config:tei || $uri || '.xml')
        let $title := ft:field($hits, 'title')
        let $targetView :=
            if (ft:field($hits, 'digital') = "true") then
                "digital"
            else
                kmg-util:getLeittextID($uri || '.xml')
        let $sort := if ($sort-parameter eq "relevance") then ft:score($hits) else $idno
        let $view := request:get-parameter("view", "edition")
        let $view-check := $view => filter(function($f as xs:string) {$f eq ft:field($hits, 'type')})
        order by
            $sort ascending
        where $view-check
        return
            search:search-results($idno, $title, $uri, $targetView, $term, $hits)
};

declare function search:search-descending($search, $term, $sort-parameter) {
    for $hits in $search
        let $doc-name := util:document-name($hits)
        let $uri := ft:field($hits, 'uri')
        let $idno := ft:field($hits, 'idno')
        let $doc := doc($config:tei || $uri || '.xml')
        let $title := ft:field($hits, 'title')
        let $targetView :=
            if (ft:field($hits, 'digital') = "true") then
                "digital"
            else
                kmg-util:getLeittextID($uri || '.xml')
        let $sort := if ($sort-parameter eq "relevance") then ft:score($hits) else $idno
        let $view := request:get-parameter("view", "edition")
        let $view-check := $view => filter(function($f as xs:string) {$f eq ft:field($hits, 'type')})
        order by
            $sort descending
        where $view-check
        return
            search:search-results($idno, $title, $uri, $targetView, $term, $hits)

};

declare function search:search-results($idno, $title, $uri, $targetView, $term, $hits) {
    element li {
        element details  {
            element summary {
                element h2 {
                    attribute class {"search-result-title"},
                    element span {
                        attribute class {"search-title-type"},
                        $idno
                    },
                    element span {
                        attribute class {"search-title"},
                        $title,
                        if (ft:field($hits, "type") = "commentary") then
                            element span {
                                " (" || lang:translate("commentary") || ")"
                            }
                        else ()
                    },
                    element span {
                        attribute aria-hidden {"true"},
                        attribute class { "fa fa-caret-down" }
                    }
                }
            },
            element ol {
                attribute class {"search-list"},
                attribute role  {"list"},
                for $hit in kwic:summarize($hits, <config width="40"/>)
                order by ft:score($hits) descending
                return
                    element li {
                        attribute class {"search-result-hit"},
                        element a {
                            attribute class {"search-result"},
                            attribute href {"kmg.html?file="
                                || $uri
                                || "&amp;edition="
                                || $targetView
                                || "&amp;search="
                                || $term
                                },
                            attribute target {"_blank"},
                            element span {
                                attribute class {"fa fa-arrow-right"}
                            },
                            (: hits are returned as html:p, but we only want the contained spans :)
                            $hit/*
                        }
                    }
                }
        }
        }
};

declare function search:search($term as xs:string)
{
    let $facets := search:facet-map()
    let $options :=
        map {
            "facets": $facets,
            (: "default-operator": "and" :)
            "fields": ("title", "idno", "uri", "digital", "type", "mainText", "varium")
        }
    let $query := $term
        (: element query {
            element phrase { $term }
        } :)
    let $view := request:get-parameter("view", "edition")
    return
        collection('/db/data/sade/resources/tei')//tei:text[@type = $view][ft:query(., $query, $options)]
};

(:~
 : This function maps the search parameters to the relevant facets.
 : This is mostly straightforward, but the "view" and "main-wit" parameters requires some explicit handling,
 : as it is presented not as a facet, but as a set of options in the front end.
 :)
declare function search:facet-map()
as map(*)
{
    let $genre := request:get-parameter("genre", "") => tokenize("\+")
    let $keyword := request:get-parameter("keyword", "") => tokenize("\+")
    return
    map {
        "genre": $genre,
        "keyword": $keyword
    }
};

declare function search:get-sorting() {
    request:get-parameter("sort", "relevance")
};